package com.me.pacman.state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.me.pacman.PacDude;
import com.me.pacman.entity.MovableEntity;
import com.me.pacman.entity.Pacman;
import com.me.pacman.level.Level;
import com.me.pacman.level.LevelTile;

public class PlayState extends LevelState {

    private Texture levelBackground;
    private BitmapFont font;

    private int pelletCount;
    private int pelletEatenCount;
    private int score;
    private int lives;
    private int round;
    public boolean paused = false;
    public boolean started = false;

    private long sirenId;

    private float waitTimer;

    private TextureRegion lifeSprite;

    private Pacman pacman;

    public PlayState(PacDude game) {
        super(game);
    }

    @Override
    public void setup() {
        levelBackground = game.assets.getLevelBackground();
        font = game.assets.getFont();
        lifeSprite = game.assets.pacman[2][1];
        Gdx.input.setInputProcessor(new Controller());

        pelletCount = 0;
        pelletEatenCount = 0;
        score = 0;
        lives = 3;
        round = 0;

        newGame();
    }

    @Override
    public void render() {
        game.batch.draw(levelBackground, 0, 16);
        level.render(0, 16);

        pacman.render(game.batch, 0, 16);

        for (int i = 0; i < lives; i++) {
            game.batch.draw(lifeSprite, i * 16, 0);
        }

        game.assets.getFont().draw(game.batch, "" + score, 40, 279);
    }

    @Override
    public void update(float dt) {
        if (waitTimer > 0) {
            waitTimer -= dt;
            if (waitTimer <= 0) {
                startGame();
            } else {
                return;
            }
        }

        pacman.update(dt);
    }

    public void newGame() {
        round++;
        game.assets.siren.stop(sirenId);

        level = new Level(game,"level");
        pelletCount = level.getTileCount(LevelTile.PELLET);
        pelletCount += level.getTileCount(LevelTile.POWER_PELLET);

        if (round == 1) {
            waitTimer = 4.3f;
            game.assets.beginning.play(1.0f);
        } else {
            waitTimer = 4.2f;
            game.assets.beginning_alt.play(1.0f);
        }

        pacman = new Pacman(this, false);
    }

    public void startGame() {
        pacman.moving = true;
        game.assets.beginning.stop();
        game.assets.beginning_alt.stop();
        sirenId = game.assets.siren.loop(1.0f);
        if (round == 1) {
            lives--;
        }
    }

    private void pelletEaten(float x, float y) {
        level.setTile(x, y, LevelTile.EMPTY);
        if (pelletEatenCount % 2 == 0) {
            game.assets.chomp_1.play(1.0f);
        } else {
            game.assets.chomp_2.play(1.0f);
        }
        pelletEatenCount++;
        pelletCount--;

        if (pelletCount == 0) {
            newGame();
        }
    }

    public void eatPellet(float x, float y) {
        pelletEaten(x, y);
        score += 10;
    }

    public void eatPowerPellet(float x, float y) {
        pelletEaten(x, y);
        // TODO: Enable ghost chase mode
        score += 50;
    }

    @Override
    public void dispose() {
        Gdx.input.setInputProcessor(null);
    }

    private final class Controller extends InputAdapter {

        @Override
        public boolean keyDown(int keycode) {
            switch (keycode) {
                case Input.Keys.UP:
                    pacman.nextDirection = MovableEntity.Direction.NORTH;
                    break;
                case Input.Keys.DOWN:
                    pacman.nextDirection = MovableEntity.Direction.SOUTH;
                    break;
                case Input.Keys.LEFT:
                    pacman.nextDirection = MovableEntity.Direction.WEST;
                    break;
                case Input.Keys.RIGHT:
                    pacman.nextDirection = MovableEntity.Direction.EAST;
                    break;
            }
            return super.keyDown(keycode);
        }

    }

}
